#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h>
#include <limits.h>
#include <misc.h>
#include <dialog.h>
#include <translat.h>
#include "tlmplib.h"
#include "tlmplib.m"

class TLMPARG: public ARRAY_OBJ{
public:
	char letter;
	const char *name;
	const char *desc;
	enum ARG_TYPE{
		ARG_PTSTR,ARG_STRING,ARG_INT,ARG_BOOL,ARG_STRINGS
	} type;
	union {
		const char **valptstr;
		SSTRING *valstr;
		SSTRINGS *valstrs;
		int *valint;
		bool *valbool;
	};
	bool mandatory;
	bool seen;
	void init ()
	{
		seen = false;
	}
	TLMPARG(char _letter, const char *_name, const char *_desc, const char *&_val, bool _mandatory)
	{
		init();
		mandatory = _mandatory;
		letter = _letter;
		name = _name;
		desc = _desc;
		valptstr = &_val;
		type = ARG_PTSTR;
	}
	TLMPARG(char _letter, const char *_name, const char *_desc, SSTRING &_val, bool _mandatory)
	{
		init();
		mandatory = _mandatory;
		letter = _letter;
		name = _name;
		desc = _desc;
		valstr = &_val;
		type = ARG_STRING;
	}
	TLMPARG(char _letter, const char *_name, const char *_desc, SSTRINGS &_vals, bool _mandatory)
	{
		init();
		mandatory = _mandatory;
		letter = _letter;
		name = _name;
		desc = _desc;
		valstrs = &_vals;
		type = ARG_STRINGS;
	}
	TLMPARG(char _letter, const char *_name, const char *_desc, int &_val, bool _mandatory)
	{
		init();
		mandatory = _mandatory;
		letter = _letter;
		name = _name;
		desc = _desc;
		valint = &_val;
		type = ARG_INT;
	}
	TLMPARG(char _letter, const char *_name, const char *_desc, bool &_val, bool _mandatory)
	{
		init();
		mandatory = _mandatory;
		letter = _letter;
		name = _name;
		desc = _desc;
		valbool = &_val;
		type = ARG_BOOL;
	}
};

class TLMPARGS: public ARRAY{
public:
	TLMPARG *getitem(int no) const
	{
		return (TLMPARG*)ARRAY::getitem(no);
	}
};

struct _F_tlmpprogram___v0_private{
	SSTRING progname,version,desc;
	TLMPARGS args;
};

void _F_tlmpprogram___v0::setproginfo (
	const char *name,
	const char *version,
	const char *desc)
{
	priv->progname.setfrom (name);
	priv->version.setfrom (version);
	priv->desc.setfrom (desc);
}

void _F_tlmpprogram___v0::setproginfo (
	const char *name,
	int version,
	int release,
	const char *desc)
{
	char tmp[20];
	snprintf (tmp,sizeof(tmp)-1,"%d.%d",version,release);
	setproginfo (name,tmp,desc);
}
	
void _F_tlmpprogram___v0::setarg (
	const char letter,
	const char *name,
	const char *desc,
	const char *&val,
	bool mandatory)
{
	priv->args.add (new TLMPARG(letter,name,desc,val,mandatory));
}
void _F_tlmpprogram___v0::setarg (
	const char letter,
	const char *name,
	const char *desc,
	SSTRING &val,
	bool mandatory)
{
	priv->args.add (new TLMPARG(letter,name,desc,val,mandatory));
}
void _F_tlmpprogram___v0::setarg (
	const char letter,
	const char *name,
	const char *desc,
	SSTRINGS &vals,
	bool mandatory)
{
	priv->args.add (new TLMPARG(letter,name,desc,vals,mandatory));
}

void _F_tlmpprogram___v0::setarg (
	const char letter,
	const char *name,
	const char *desc,
	int &val,
	bool mandatory)
{
	priv->args.add (new TLMPARG(letter,name,desc,val,mandatory));
}
void _F_tlmpprogram___v0::setarg (
	const char letter,
	const char *name,
	const char *desc,
	bool &val,
	bool mandatory)
{
	priv->args.add (new TLMPARG(letter,name,desc,val,mandatory));
}

void _F_tlmpprogram___v1::usage()
{
	fprintf (stderr,MSG_U(I_PROGUSAGE,"%s version %s\n\n%s\n")
		,priv->progname.get(),priv->version.get()
		,priv->desc.get());
	printf ("\n");
	for (int i=0; i<priv->args.getnb(); i++){
		TLMPARG *arg = priv->args.getitem(i);
		printf ("-%c --%s: %s %s\n",arg->letter,arg->name,arg->desc
			,arg->mandatory ? MSG_U(I_MANDATORY,"(Mandatory)") : "");
	}
	printf ("\n");
}

void _F_tlmpprogram___v1::showerror(const char *msg)
{
	fprintf (stderr,"%s",msg);
}

/*
	No argument by default
*/
int _F_tlmpprogram___v1::onearg(const char *, const char *)
{
	return -1;
}

/*
	Main called when there is no argument.
	the default call the usage function
	If a program may be called with and without argument, its main_noarg
	function just have to call main(0,NULL);
*/
int _F_tlmpprogram___v1::main_noarg()
{
	usage();
	return -1;
}
/*
	Main called when there is 1 or more arguments.
	the default call the usage function
*/
int _F_tlmpprogram___v1::main(int, char *[])
{
	usage();
	return -1;
}

void _F_tlmpprogram___v1::init()
{
}


static _F_tlmpprogram___v1 *ctx;
static void tlmpprogram_error (const char *msg)
{
	ctx->showerror (msg);
}

static int tlmpprogram_loadif (
	const char *path,
	const char *lang,
	SSTRINGS &dir,
	const char *pkg)
{
	int ret = -1;
	int nb = dir.getnb();
	int len = strlen(pkg);
	for (int i=0; i<nb; i++){
		const char *f = dir.getitem(i)->get();
		if (strncmp(pkg,f,len)==0
			&& (f[len] == '\0' || f[len] == '-')){
			translat_load (path,"LINUXCONF_DICT",f,"LINUXCONF_LANG",lang);
			ret = 0;
			break;
		}
	}
	return ret;
}

static const char *tlmpprogram_getprog(char *argv[])
{
	const char *progname = strrchr(argv[0],'/');
	if (progname == NULL){
		progname = argv[0];
	}else{
		progname++;
	}
	return progname;
}

/*
	Check for short and long name option with argument or not.
	Check against the "setarg" argument and then call the onearg functag.
*/
static int tlmpprogram_checkarg (
	_F_tlmpprogram___v1 &c,
	const TLMPARGS &args,
	const char *arg,
	const char *val)
{
	int ret = -1;
	bool missing_arg = false;
	if (val == NULL || (val[0] == '-' && val[1] == '-')){
		missing_arg = true;
		val = "";
	}
	for (int i=0; i<args.getnb(); i++){
		TLMPARG *a = args.getitem(i);
		if (a->letter == arg[1] || strcmp(a->name,arg+2)==0){
			if (a->type == TLMPARG::ARG_STRINGS){
				a->valstrs->add (new SSTRING(val));
				ret = 1;
			}else if (a->seen){
				fprintf (stderr,"Option %s can't be repeated, ignored\n",arg);
			}else if (a->type == TLMPARG::ARG_PTSTR){
				*a->valptstr = val;
				ret = 1;
			}else if (a->type == TLMPARG::ARG_BOOL){
				*a->valbool = true;
				ret = 0;
			}else if (a->type == TLMPARG::ARG_INT){
				*a->valint = atoi(val);
				ret = 1;
			}else if (a->type == TLMPARG::ARG_STRING){
				a->valstr->setfrom(val);
				ret = 1;
			}
			a->seen = true;
			break;
		}
	}
	if (ret == -1){
		ret = c.onearg (arg,val);
	}
	if (ret == 1 && missing_arg){
		fprintf (stderr,MSG_U(E_MISSINGARG,"Missing value for option %s\n"),arg);
		ret = -1;
	}
	return ret;
}


int tlmpprogram___v1 (
	_F_tlmpprogram___v1 &c,
	int argc,
	char *argv[],
	const char *packages[])
{
	_F_tlmpprogram___v0_private priv;
	c.priv = &priv;
	priv.progname.setfrom (tlmpprogram_getprog (argv));
	ctx = &c;
	diagui_seticonpath ("/usr/lib/tlmp/images");
	diagui_seticonpath ("/usr/share/icons");
	diagui_seticonpath ("/usr/share/icons/mini");
	diagui_seticonpath ("/usr/lib/linuxconf/images");
	tlmp_seterrorfct (tlmpprogram_error);
	{
		// Set the default configuration file for this program
		const char *home = getenv ("HOME");
		if (home != NULL){
			SSTRING path;
			path.setfromf("%s/.tlmp",home);
			mkdir (path.get(),0700);	// Make sure the sub-dir is created
			path.setfromf("%s/.tlmp/%s.conf",home,priv.progname.get());
			linuxconf_setdbpath (path.get());
		}
	}
	const char *lang = linuxconf_getlang();
	linuxconf_loadlibmsg ("/usr/lib/linuxconf-lib");
	// translat_load ("/usr/lib/tlmp/help","tlmp", lang);

	// Load the various dictionary files
	// Assumes all dictionnary are located in /usr/lib/tlmp/help.lang
	// or in the linuxconf project
	SSTRINGS tbtlmp,tblinuxconf;
	dir_getlist ("/usr/lib/tlmp/help.eng",".eng",tbtlmp);
	dir_getlist ("/usr/lib/linuxconf/help.eng",".eng",tblinuxconf);
	for (int i=0; packages[i] != NULL; i++){
		const char *pkg = packages[i];
		if (tlmpprogram_loadif ("/usr/lib/tlmp/help",lang,tbtlmp,pkg)==-1){
			tlmpprogram_loadif ("/usr/lib/linuxconf/help",lang,tblinuxconf,pkg);
		}
	}
	translat_checkmissing();
	c.init();
	int i;
	int errarg = 0;
	argc = dialog_parseuioptions(argc,argv);
	bool help_requested = false;
	for (i=1; i<argc; i++){
		const char *arg = argv[i];
		if (strcmp(arg,"--help")==0){
			c.usage();
			help_requested = true;
		}else if (arg[0] == '-'){
			if (arg[1] == '-' && arg[2] == '\0'){
				// This acts as a marker to end argument parsing
				i++;
				break;
			}else{
				int ok = 0;
				if (isalpha(arg[1])){
					if (arg[2] != '\0'){
						ok = tlmpprogram_checkarg (c,priv.args,arg,arg+2);
					}else{
						ok = tlmpprogram_checkarg (c,priv.args,arg,argv[i+1]);
						if (ok == 1) i++;
					}
				}else if(arg[1] == '-'){
					ok = tlmpprogram_checkarg (c,priv.args,arg,argv[i+1]);
					if (ok == 1) i++;
				}
				if (ok == -1){
					fprintf (stderr,MSG_U(E_IVLDARG,"Invalid argument: %s\n"),arg);
					errarg++;
				}
			}
		}else{
			break;
		}
	}
	int ret = -1;
	if (help_requested){
		ret = 0;
	}else{
		for (int a=0; a<priv.args.getnb(); a++){
			TLMPARG *pta = priv.args.getitem(a);
			if (pta->mandatory && !pta->seen){
				errarg++;
				fprintf (stderr,MSG_U(E_NOTSPECIFIED,"Option %s is mandatory\n")
					,pta->name);
			}
		}
		if (errarg > 0){
			c.usage();
		}else if (argc == i){
			ret = c.main_noarg();
		}else{
			ret = c.main (argc-i,argv+i);
		}
	}
	tlmp_seterrorfct (NULL);
	return ret;
}

int tlmpprogram___v1 (
	_F_tlmpprogram___v1 &c,
	int argc,
	char *argv[])
{
	static const char *packages[]={NULL};
	return tlmpprogram___v1 (c,argc,argv,packages);
}

int tlmpprogram___v1 (
	_F_tlmpprogram___v1 &c,
	int argc,
	char *argv[],
	const char *package)
{
	static const char *packages[]={package,NULL};
	return tlmpprogram___v1 (c,argc,argv,packages);
}

